@@ -118,7 +118,7 @@ def ListJobs(opts, args):
 
   qfilter = qlang.MakeSimpleFilter("status", opts.status_filter)
 
-  cl = GetClient(query=True)
+  cl = GetClient()
 
   return GenericList(constants.QR_JOB, selected_fields, args, None,
                      opts.separator, not opts.no_headers,
@@ -137,7 +137,7 @@ def ListJobFields(opts, args):
   @return: the desired exit code
 
   """
-  cl = GetClient(query=True)
+  cl = GetClient()
 
   return GenericListFields(constants.QR_JOB, args, opts.separator,
                            not opts.no_headers, cl=cl)
@@ -285,6 +285,40 @@ def ChangePriority(opts, args):
                            cl.ChangeJobPriority(job_id, opts.priority))
 
 
+def _ListOpcodeTimestamp(name, ts, container):
+  """ Adds the opcode timestamp to the given container.
+
+  """
+  if isinstance(ts, (tuple, list)):
+    container.append((name, FormatTimestamp(ts), "opcode_timestamp"))
+  else:
+    container.append((name, "N/A", "opcode_timestamp"))
+
+
+def _CalcDelta(from_ts, to_ts):
+  """ Calculates the delta between two timestamps.
+
+  """
+  return to_ts[0] - from_ts[0] + (to_ts[1] - from_ts[1]) / 1000000.0
+
+
+def _ListJobTimestamp(name, ts, container, prior_ts=None):
+  """ Adds the job timestamp to the given container.
+
+  @param prior_ts: The timestamp used to calculate the amount of time that
+                   passed since the given timestamp.
+
+  """
+  if ts is not None:
+    delta = ""
+    if prior_ts is not None:
+      delta = " (delta %.6fs)" % _CalcDelta(prior_ts, ts)
+    output = "%s%s" % (FormatTimestamp(ts), delta)
+    container.append((name, output, "job_timestamp"))
+  else:
+    container.append((name, "unknown (%s)" % str(ts), "job_timestamp"))
+
+
 def ShowJobs(opts, args):
   """Show detailed information about jobs.
 
@@ -295,131 +329,83 @@ def ShowJobs(opts, args):
   @return: the desired exit code
 
   """
-  def format_msg(level, text):
-    """Display the text indented."""
-    ToStdout("%s%s", "  " * level, text)
-
-  def result_helper(value):
-    """Format a result field in a nice way."""
-    if isinstance(value, (tuple, list)):
-      return "[%s]" % utils.CommaJoin(value)
-    else:
-      return str(value)
-
   selected_fields = [
     "id", "status", "ops", "opresult", "opstatus", "oplog",
     "opstart", "opexec", "opend", "received_ts", "start_ts", "end_ts",
     ]
 
   qfilter = qlang.MakeSimpleFilter("id", _ParseJobIds(args))
-  cl = GetClient(query=True)
+  cl = GetClient()
   result = cl.Query(constants.QR_JOB, selected_fields, qfilter).data
 
-  first = True
+  job_info_container = []
 
   for entry in result:
-    if not first:
-      format_msg(0, "")
-    else:
-      first = False
-
     ((_, job_id), (rs_status, status), (_, ops), (_, opresult), (_, opstatus),
      (_, oplog), (_, opstart), (_, opexec), (_, opend), (_, recv_ts),
      (_, start_ts), (_, end_ts)) = entry
 
     # Detect non-normal results
     if rs_status != constants.RS_NORMAL:
-      format_msg(0, "Job ID %s not found" % job_id)
+      job_info_container.append("Job ID %s not found" % job_id)
       continue
 
-    format_msg(0, "Job ID: %s" % job_id)
+    # Container for produced data
+    job_info = [("Job ID", job_id)]
+
     if status in _USER_JOB_STATUS:
       status = _USER_JOB_STATUS[status]
     else:
       raise errors.ProgrammerError("Unknown job status code '%s'" % status)
 
-    format_msg(1, "Status: %s" % status)
+    job_info.append(("Status", status))
 
-    if recv_ts is not None:
-      format_msg(1, "Received:         %s" % FormatTimestamp(recv_ts))
-    else:
-      format_msg(1, "Missing received timestamp (%s)" % str(recv_ts))
-
-    if start_ts is not None:
-      if recv_ts is not None:
-        d1 = start_ts[0] - recv_ts[0] + (start_ts[1] - recv_ts[1]) / 1000000.0
-        delta = " (delta %.6fs)" % d1
-      else:
-        delta = ""
-      format_msg(1, "Processing start: %s%s" %
-                 (FormatTimestamp(start_ts), delta))
-    else:
-      format_msg(1, "Processing start: unknown (%s)" % str(start_ts))
-
-    if end_ts is not None:
-      if start_ts is not None:
-        d2 = end_ts[0] - start_ts[0] + (end_ts[1] - start_ts[1]) / 1000000.0
-        delta = " (delta %.6fs)" % d2
-      else:
-        delta = ""
-      format_msg(1, "Processing end:   %s%s" %
-                 (FormatTimestamp(end_ts), delta))
-    else:
-      format_msg(1, "Processing end:   unknown (%s)" % str(end_ts))
+    _ListJobTimestamp("Received", recv_ts, job_info)
+    _ListJobTimestamp("Processing start", start_ts, job_info, prior_ts=recv_ts)
+    _ListJobTimestamp("Processing end", end_ts, job_info, prior_ts=start_ts)
 
     if end_ts is not None and recv_ts is not None:
-      d3 = end_ts[0] - recv_ts[0] + (end_ts[1] - recv_ts[1]) / 1000000.0
-      format_msg(1, "Total processing time: %.6f seconds" % d3)
+      job_info.append(("Total processing time", "%.6f seconds" %
+                       _CalcDelta(recv_ts, end_ts)))
     else:
-      format_msg(1, "Total processing time: N/A")
-    format_msg(1, "Opcodes:")
+      job_info.append(("Total processing time", "N/A"))
+
+    opcode_container = []
     for (opcode, result, status, log, s_ts, x_ts, e_ts) in \
             zip(ops, opresult, opstatus, oplog, opstart, opexec, opend):
-      format_msg(2, "%s" % opcode["OP_ID"])
-      format_msg(3, "Status: %s" % status)
-      if isinstance(s_ts, (tuple, list)):
-        format_msg(3, "Processing start: %s" % FormatTimestamp(s_ts))
-      else:
-        format_msg(3, "No processing start time")
-      if isinstance(x_ts, (tuple, list)):
-        format_msg(3, "Execution start:  %s" % FormatTimestamp(x_ts))
-      else:
-        format_msg(3, "No execution start time")
-      if isinstance(e_ts, (tuple, list)):
-        format_msg(3, "Processing end:   %s" % FormatTimestamp(e_ts))
-      else:
-        format_msg(3, "No processing end time")
-      format_msg(3, "Input fields:")
-      for key in utils.NiceSort(opcode.keys()):
-        if key == "OP_ID":
-          continue
-        val = opcode[key]
-        if isinstance(val, (tuple, list)):
-          val = ",".join([str(item) for item in val])
-        format_msg(4, "%s: %s" % (key, val))
-      if result is None:
-        format_msg(3, "No output data")
-      elif isinstance(result, (tuple, list)):
-        if not result:
-          format_msg(3, "Result: empty sequence")
-        else:
-          format_msg(3, "Result:")
-          for elem in result:
-            format_msg(4, result_helper(elem))
-      elif isinstance(result, dict):
-        if not result:
-          format_msg(3, "Result: empty dictionary")
-        else:
-          format_msg(3, "Result:")
-          for key, val in result.iteritems():
-            format_msg(4, "%s: %s" % (key, result_helper(val)))
-      else:
-        format_msg(3, "Result: %s" % result)
-      format_msg(3, "Execution log:")
+      opcode_info = []
+      opcode_info.append(("Opcode", opcode["OP_ID"]))
+      opcode_info.append(("Status", status))
+
+      _ListOpcodeTimestamp("Processing start", s_ts, opcode_info)
+      _ListOpcodeTimestamp("Execution start", x_ts, opcode_info)
+      _ListOpcodeTimestamp("Processing end", e_ts, opcode_info)
+
+      opcode_info.append(("Input fields", opcode))
+      opcode_info.append(("Result", result))
+
+      exec_log_container = []
       for serial, log_ts, log_type, log_msg in log:
         time_txt = FormatTimestamp(log_ts)
         encoded = FormatLogMessage(log_type, log_msg)
-        format_msg(4, "%s:%s:%s %s" % (serial, time_txt, log_type, encoded))
+
+        # Arranged in this curious way to preserve the brevity for multiple
+        # logs. This content cannot be exposed as a 4-tuple, as time contains
+        # the colon, causing some YAML parsers to fail.
+        exec_log_info = [
+          ("Time", time_txt),
+          ("Content", (serial, log_type, encoded,)),
+          ]
+        exec_log_container.append(exec_log_info)
+      opcode_info.append(("Execution log", exec_log_container))
+
+      opcode_container.append(opcode_info)
+
+    job_info.append(("Opcodes", opcode_container))
+    job_info_container.append(job_info)
+
+  PrintGenericInfo(job_info_container)
+
   return 0
 
 
