@@ -154,7 +154,7 @@ def _ParseInstanceList(lines, include_node):
   return result
 
 
-def _GetAllInstanceList(fn, include_node, _timeout=5):
+def _GetAllInstanceList(fn, include_node, delays, timeout):
   """Return the list of instances including running and shutdown.
 
   See L{_RunInstanceList} and L{_ParseInstanceList} for parameter details.
@@ -162,7 +162,7 @@ def _GetAllInstanceList(fn, include_node, _timeout=5):
   """
   instance_list_errors = []
   try:
-    lines = utils.Retry(_RunInstanceList, (0.3, 1.5, 1.0), _timeout,
+    lines = utils.Retry(_RunInstanceList, delays, timeout,
                         args=(fn, instance_list_errors))
   except utils.RetryTimeout:
     if instance_list_errors:
@@ -182,7 +182,7 @@ def _IsInstanceRunning(instance_info):
   """Determine whether an instance is running.
 
   An instance is running if it is in the following Xen states:
-  running, blocked, or paused.
+  running, blocked, paused, or dying (about to be destroyed / shutdown).
 
   For some strange reason, Xen once printed 'rb----' which does not make any
   sense because an instance cannot be both running and blocked.  Fortunately,
@@ -193,6 +193,9 @@ def _IsInstanceRunning(instance_info):
   to be scheduled to run.
   http://old-list-archives.xenproject.org/xen-users/2007-06/msg00849.html
 
+  A dying instance is about to be removed, but it is still consuming resources,
+  and counts as running.
+
   @type instance_info: string
   @param instance_info: Information about instance, as supplied by Xen.
   @rtype: bool
@@ -202,15 +205,51 @@ def _IsInstanceRunning(instance_info):
   return instance_info == "r-----" \
       or instance_info == "rb----" \
       or instance_info == "-b----" \
-      or instance_info == "--p---" \
+      or instance_info == "-----d" \
       or instance_info == "------"
 
 
 def _IsInstanceShutdown(instance_info):
-  return instance_info == "---s--"
+  """Determine whether the instance is shutdown.
+
+  An instance is shutdown when a user shuts it down from within, and we do not
+  remove domains to be able to detect that.
+
+  The dying state has been added as a precaution, as Xen's status reporting is
+  weird.
+
+  """
+  return instance_info == "---s--" \
+      or instance_info == "---s-d"
+
+
+def _IgnorePaused(instance_info):
+  """Removes information about whether a Xen state is paused from the state.
+
+  As it turns out, an instance can be reported as paused in almost any
+  condition. Paused instances can be paused, running instances can be paused for
+  scheduling, and any other condition can appear to be paused as a result of
+  races or improbable conditions in Xen's status reporting.
+  As we do not use Xen's pause commands in any way at the time, we can simply
+  ignore the paused field and save ourselves a lot of trouble.
+
+  Should we ever use the pause commands, several samples would be needed before
+  we could confirm the domain as paused.
+
+  """
+  return instance_info.replace('p', '-')
 
 
 def _XenToHypervisorInstanceState(instance_info):
+  """Maps Xen states to hypervisor states.
+
+  @type instance_info: string
+  @param instance_info: Information about instance, as supplied by Xen.
+  @rtype: L{hv_base.HvInstanceState}
+
+  """
+  instance_info = _IgnorePaused(instance_info)
+
   if _IsInstanceRunning(instance_info):
     return hv_base.HvInstanceState.RUNNING
   elif _IsInstanceShutdown(instance_info):
@@ -221,23 +260,23 @@ def _XenToHypervisorInstanceState(instance_info):
                                    instance_info)
 
 
-def _GetRunningInstanceList(fn, include_node, _timeout=5):
+def _GetRunningInstanceList(fn, include_node, delays, timeout):
   """Return the list of running instances.
 
   See L{_GetAllInstanceList} for parameter details.
 
   """
-  instances = _GetAllInstanceList(fn, include_node, _timeout)
+  instances = _GetAllInstanceList(fn, include_node, delays, timeout)
   return [i for i in instances if hv_base.HvInstanceState.IsRunning(i[4])]
 
 
-def _GetShutdownInstanceList(fn, include_node, _timeout=5):
+def _GetShutdownInstanceList(fn, include_node, delays, timeout):
   """Return the list of shutdown instances.
 
   See L{_GetAllInstanceList} for parameter details.
 
   """
-  instances = _GetAllInstanceList(fn, include_node, _timeout)
+  instances = _GetAllInstanceList(fn, include_node, delays, timeout)
   return [i for i in instances if hv_base.HvInstanceState.IsShutdown(i[4])]
 
 
@@ -424,6 +463,9 @@ class XenHypervisor(hv_base.BaseHypervisor):
   _NICS_DIR = _ROOT_DIR + "/nic" # contains NICs' info
   _DIRS = [_ROOT_DIR, _NICS_DIR]
 
+  _INSTANCE_LIST_DELAYS = (0.3, 1.5, 1.0)
+  _INSTANCE_LIST_TIMEOUT = 5
+
   ANCILLARY_FILES = [
     XEND_CONFIG_FILE,
     XL_CONFIG_FILE,
@@ -588,29 +630,32 @@ class XenHypervisor(hv_base.BaseHypervisor):
           (nic_type, constants.HT_HVM_VIF_IOEMU)
 
     for idx, nic in enumerate(instance.nics):
-      nic_str = "mac=%s%s" % (nic.mac, nic_type_str)
-
-      ip = getattr(nic, "ip", None)
-      if ip is not None:
-        nic_str += ", ip=%s" % ip
-
-      if nic.nicparams[constants.NIC_MODE] == constants.NIC_MODE_BRIDGED:
-        nic_str += ", bridge=%s" % nic.nicparams[constants.NIC_LINK]
-      elif nic.nicparams[constants.NIC_MODE] == constants.NIC_MODE_OVS:
-        nic_str += ", bridge=%s" % nic.nicparams[constants.NIC_LINK]
-        if nic.nicparams[constants.NIC_VLAN]:
-          nic_str += "%s" % nic.nicparams[constants.NIC_VLAN]
+      nic_args = {}
+      nic_args["mac"] = "%s%s" % (nic.mac, nic_type_str)
 
       if nic.name and \
             nic.name.startswith(constants.INSTANCE_COMMUNICATION_NIC_PREFIX):
         tap = hv_base.GenerateTapName()
-        nic_str += ", vifname=%s" % tap
+        nic_args["vifname"] = tap
+        nic_args["script"] = pathutils.XEN_VIF_METAD_SETUP
         nic.name = tap
+      else:
+        ip = getattr(nic, "ip", None)
+        if ip is not None:
+          nic_args["ip"] = ip
 
-      if hvp[constants.HV_VIF_SCRIPT]:
-        nic_str += ", script=%s" % hvp[constants.HV_VIF_SCRIPT]
+        if nic.nicparams[constants.NIC_MODE] == constants.NIC_MODE_BRIDGED:
+          nic_args["bridge"] = nic.nicparams[constants.NIC_LINK]
+        elif nic.nicparams[constants.NIC_MODE] == constants.NIC_MODE_OVS:
+          nic_args["bridge"] = nic.nicparams[constants.NIC_LINK]
+          if nic.nicparams[constants.NIC_VLAN]:
+            nic_args["bridge"] += nic.nicparams[constants.NIC_VLAN]
 
-      vif_data.append("'%s'" % nic_str)
+        if hvp[constants.HV_VIF_SCRIPT]:
+          nic_args["script"] = hvp[constants.HV_VIF_SCRIPT]
+
+      nic_str = ", ".join(["%s=%s" % p for p in nic_args.items()])
+      vif_data.append("'%s'" % (nic_str, ))
       self._WriteNICInfoFile(instance, idx, nic)
 
     config.write("vif = [%s]\n" % ",".join(vif_data))
@@ -674,7 +719,8 @@ class XenHypervisor(hv_base.BaseHypervisor):
 
     """
     return _GetAllInstanceList(lambda: self._RunXen(["list"], hvparams),
-                               include_node)
+                               include_node, delays=self._INSTANCE_LIST_DELAYS,
+                               timeout=self._INSTANCE_LIST_TIMEOUT)
 
   def ListInstances(self, hvparams=None):
     """Get the list of running instances.
@@ -688,7 +734,8 @@ class XenHypervisor(hv_base.BaseHypervisor):
     """
     instance_list = _GetRunningInstanceList(
       lambda: self._RunXen(["list"], hvparams),
-      False)
+      False, delays=self._INSTANCE_LIST_DELAYS,
+      timeout=self._INSTANCE_LIST_TIMEOUT)
     return [info[0] for info in instance_list]
 
   def GetInstanceInfo(self, instance_name, hvparams=None):
