@@ -39,6 +39,7 @@ The module is also used by the master daemon.
 """
 
 from ganeti import constants
+from ganeti import pathutils
 from ganeti import objects
 import ganeti.rpc.client as cl
 from ganeti.rpc.errors import RequestError
@@ -90,9 +91,17 @@ class Client(cl.AbstractClient):
     Arguments are the same as for L{AbstractClient}.
 
     """
-    super(Client, self).__init__(address, timeouts, transport)
+    super(Client, self).__init__(timeouts, transport)
     # Override the version of the protocol:
     self.version = constants.LUXI_VERSION
+    # Store the socket address
+    if address is None:
+      address = pathutils.QUERY_SOCKET
+    self.address = address
+    self._InitTransport()
+
+  def _GetAddress(self):
+    return self.address
 
   def SetQueueDrainFlag(self, drain_flag):
     return self.CallMethod(REQ_SET_DRAIN_FLAG, (drain_flag, ))
@@ -104,7 +113,9 @@ class Client(cl.AbstractClient):
     return self.CallMethod(REQ_PICKUP_JOB, (job,))
 
   def SubmitJob(self, ops):
-    ops_state = map(lambda op: op.__getstate__(), ops)
+    ops_state = map(lambda op: op.__getstate__()
+                               if not isinstance(op, objects.ConfigObject)
+                               else op.ToDict(_with_private=True), ops)
     return self.CallMethod(REQ_SUBMIT_JOB, (ops_state, ))
 
   def SubmitJobToDrainedQueue(self, ops):
