@@ -77,8 +77,8 @@ from ganeti import jstore
 from ganeti.hypervisor import hv_base
 
 from ganeti.constants import (QFT_UNKNOWN, QFT_TEXT, QFT_BOOL, QFT_NUMBER,
-                              QFT_UNIT, QFT_TIMESTAMP, QFT_OTHER,
-                              RS_NORMAL, RS_UNKNOWN, RS_NODATA,
+                              QFT_NUMBER_FLOAT, QFT_UNIT, QFT_TIMESTAMP,
+                              QFT_OTHER, RS_NORMAL, RS_UNKNOWN, RS_NODATA,
                               RS_UNAVAIL, RS_OFFLINE)
 
 (NETQ_CONFIG,
@@ -136,6 +136,7 @@ _VERIFY_FN = {
   QFT_TEXT: ht.TString,
   QFT_BOOL: ht.TBool,
   QFT_NUMBER: ht.TInt,
+  QFT_NUMBER_FLOAT: ht.TFloat,
   QFT_UNIT: ht.TInt,
   QFT_TIMESTAMP: ht.TNumber,
   QFT_OTHER: lambda _: True,
@@ -163,6 +164,7 @@ _VTToQFT = {
   constants.VTYPE_BOOL: QFT_BOOL,
   constants.VTYPE_SIZE: QFT_UNIT,
   constants.VTYPE_INT: QFT_NUMBER,
+  constants.VTYPE_FLOAT: QFT_NUMBER_FLOAT,
   }
 
 _SERIAL_NO_DOC = "%s object serial number, incremented on each modification"
@@ -2476,6 +2478,9 @@ class OsInfo(objects.ConfigObject):
     "api_versions",
     "parameters",
     "node_status",
+    "os_hvp",
+    "osparams",
+    "trusted"
     ]
 
 
@@ -2508,6 +2513,15 @@ def _BuildOsFields():
     (_MakeField("node_status", "NodeStatus", QFT_OTHER,
                 "Status from node"),
      None, 0, _GetItemAttr("node_status")),
+    (_MakeField("os_hvp", "OsHypervisorParams", QFT_OTHER,
+                "Operating system specific hypervisor parameters"),
+     None, 0, _GetItemAttr("os_hvp")),
+    (_MakeField("osparams", "OsParameters", QFT_OTHER,
+                "Operating system specific parameters"),
+     None, 0, _GetItemAttr("osparams")),
+    (_MakeField("trusted", "Trusted", QFT_BOOL,
+                "Whether this OS is trusted"),
+     None, 0, _GetItemAttr("trusted")),
     ]
 
   return _PrepareFieldList(fields, [])
