@@ -159,7 +159,7 @@ main opts clusters = do
                 "t_disk" "f_disk" "Score"
 
   when (null clusters) $ do
-         def_socket <- Path.defaultMasterSocket
+         def_socket <- Path.defaultQuerySocket
          let lsock = fromMaybe def_socket (optLuxi opts)
          let name = local
          input_data <- Luxi.loadData lsock
