@@ -35,8 +35,6 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 module Ganeti.Query.Group
   (fieldsMap) where
 
-import qualified Data.Map as Map
-
 import Ganeti.Config
 import Ganeti.Objects
 import Ganeti.Query.Language
@@ -92,5 +90,4 @@ groupFields =
 
 -- | The group fields map.
 fieldsMap :: FieldMap NodeGroup NoDataRuntime
-fieldsMap =
-  Map.fromList $ map (\v@(f, _, _) -> (fdefName f, v)) groupFields
+fieldsMap = fieldListToFieldMap groupFields
