@@ -495,4 +495,4 @@ main _ _ (server, cref, jq) = do
 
   finally
     (forever $ U.listener (luxiHandler (qlock, jq, creader)) server)
-    (closeServer server)
+    (closeServer server >> removeFile qlockFile)
