@@ -40,6 +40,7 @@ module Test.Ganeti.Query.Instance
 
 import qualified Data.Map as Map
 import qualified Data.Set as Set
+import System.Time (ClockTime(..))
 
 import Ganeti.JSON
 import Ganeti.Objects
@@ -60,9 +61,10 @@ createInstance name pnodeUuid adminState adminStateSource =
   Instance name pnodeUuid "" Kvm
     (GenericContainer Map.empty)
     (PartialBeParams Nothing Nothing Nothing Nothing Nothing Nothing)
-    (GenericContainer Map.empty)
-    adminState adminStateSource [] [] DTDrbd8 False Nothing 0.0 0.0 "" 0
-    Set.empty
+    (GenericContainer Map.empty) (GenericContainer Map.empty)
+    adminState adminStateSource [] [] DTDrbd8 False Nothing epochTime epochTime
+    "" 0 Set.empty
+  where epochTime = TOD 0 0
 
 -- | A fake InstanceInfo to be used to check values.
 fakeInstanceInfo :: InstanceInfo
